P2J.modules.puzzle=function() {
	var puzzle=$('#puzzle');
	var pict=$('#pict');
	var moving=$('#moving');
	var board=$('#board');
	var puzzleDone=$('#puzzleDone');
	var dropSound=null;
	var taDa=null;
	var moved=null;
	var lastOver=null;
	var pieces=null;

	var reload = true;	
	var pictURL = 'images/default.jpg';
	var sounds = true;
	var showBackgroundImage = true;
	var cols = 2;
	var rows = 2;
	var rotations = 0;
	var startTime=0;//milisegons en els que s'escomenca
	var elapsedTime=0;//milisegons que portem
	
	var doTapHold=false;

	var puzzleChange = function(event,config){
		pictURL=config.pictURL;
		cols=parseInt(config.cols);
		rows=parseInt(config.rows);
		rotations=parseFloat(config.rotations);
		elapsedTime=0;
		startTime=0;
		reload=true;
		if(document.location.href.indexOf("#puzzle")>0){
			pageBeforeShow();
			pageShow();
		}
	};
	
	var configChange=function(event,config){
		showBackgroundImage=config.showBackgroundImage;
		pict.toggleClass('no-background',!showBackgroundImage);
		sounds=config.sounds;
	};

	var getOverPiece = function(x,y){
		var piece=null;
		$.each(pieces,function(i,p){
			if(p.offsetLeft<=x && (p.offsetLeft+p.clientWidth)>=x && p.offsetTop<=y && (p.offsetTop+p.clientHeight)>=y){
				piece=p;
				return false;
			}
		});
		return piece;
	};

	var isOrdered=function(piece){
		var ordered=!piece.hasClass('rotated');
		var prev=piece.prev();
		while (prev.length && ordered){
			if(!prev.hasClass('rotated') && ((prev.attr('data-row')==piece.attr('data-row') && prev.attr('data-col')==piece.attr('data-col')-1) || (prev.attr('data-row')==piece.attr('data-row')-1 && prev.attr('data-col')==cols-1))){
				piece=prev;
				prev=piece.prev();
			}else ordered=false;
		}
		ordered=ordered && !piece.hasClass('rotated');
		if(ordered){
			var next=piece.next();
			while (next.length && ordered){
				if(!next.hasClass('rotated') && ((next.attr('data-row')==piece.attr('data-row') && next.attr('data-col')==parseInt(piece.attr('data-col'))+1) || (next.attr('data-row')==parseInt(piece.attr('data-row'))+1 && next.attr('data-col')==0))){
					piece=next;
					next=piece.next();
				}else ordered=false;
			}
		}
		return ordered;
	};

	var createPuzzle=function(){
		var elems=[];
		for(var row=0;row<rows;row++) for(var col=0;col<cols;col++)
			elems.push({row:row,col:col});
		
		var numRotations=Math.ceil(rows*cols*rotations);
		elems.sort(function(a,b) {
			var r=Math.round(Math.random());
			if(numRotations>0){
				if(r && !b.rotated){
					b.rotated=true;
					numRotations--;
				}else if(!a.rotated){
					a.rotated=true;
					numRotations--;
				}
			}
			return (r-0.5);
		});
		var i=0;
		while(numRotations>0 && i<elems.length){
			if(!elems[i].rotated){
				elems[i].rotated=true;
				numRotations--;
			}
			i++;
		}

		board.empty();
		for(var i=0;i<elems.length;i++)
			board.append('<div data-row="'+elems[i].row+'" data-col="'+elems[i].col+'" '+(elems[i].rotated?'class="rotated"':'')+'></div>');
		pieces=$("#board div");
		if(!isOrdered($(pieces[0]))){
			pieces.bind("touchstart", touchStart);
			pieces.bind("touchmove", touchMove);
			pieces.bind("touchend", touchEnd);
			pieces.bind("tap", tap);
			pieces.bind("taphold", tapHold);
			pieces.bind('dragover',dragOverImage);
			pieces.bind('drop',dropImage);
			
			moving.css({
				left: '110%',
				top: '110%' 
			});
			puzzle.removeClass('holded');
		}else createPuzzle();
	};

	var touchStart = function(e) {
		e.preventDefault();
		moved=$(e.originalEvent.target);
		moving.attr('data-col',moved.attr('data-col')).attr('data-row',moved.attr('data-row'));
		moving.toggleClass('rotated',moved.hasClass('rotated'));
		moving.css({
			left: moved.position().left,
			top: moved.position().top
		});
		moved.addClass('moved');
	};

	var touchMove = function(e) {  
		if(doTapHold) return;
		e.preventDefault();  
		var orig = e.originalEvent.targetTouches[0];
		var x=orig.clientX;
		var y=orig.clientY;
	  	
		moving.css({
			left: x-(moving[0].clientWidth/2),
			top: y-(moving[0].clientHeight/2) 
		});
	
		if(lastOver) lastOver.toggleClass('over');
		lastOver=$(getOverPiece(x,y)).toggleClass('over');
	};

	var touchEnd = function(e) {
		if(doTapHold) return;
		e.preventDefault();
		moved.removeClass('moved');
		moving.css({
			left: '110%',
			top: '110%' 
		});
		if(lastOver){
			if(sounds) dropSound.play();
			lastOver.removeClass('over');
			lastOver.toggleClass('rotated',moved.hasClass('rotated'));
			moved.attr('data-col',lastOver.attr('data-col')).attr('data-row',lastOver.attr('data-row'));
			lastOver.attr('data-col',moving.attr('data-col')).attr('data-row',moving.attr('data-row'));
		}
		lastOver=null;
		setTimeout(function(){
			if(isOrdered(moved)){
				elapsedTime+=new Date().getTime()-startTime;
				if(sounds) taDa.play();
				$(window).trigger('puzzleDone',{
					size : cols*rows,
					rotations : rotations,
					time : elapsedTime
				});
				reload=true;
				startTime=0;
				elapsedTime=0;
				puzzleDone.popup("open");
			}
		},500);
	};

	var tap=function(e){
		if(doTapHold){
			doTapHold=false;
			moved.removeClass('moved');
			moving.css({
				left: '110%',
				top: '110%' 
			});
			puzzle.removeClass('holded');
		}else if(rotations) $(e.originalEvent.target).toggleClass('rotated');
	};
	
	var tapHold=function(e){
		doTapHold=true;
		puzzle.addClass('holded');
	};
	
	var puzzleDoneAfterClose=function(){
		$.mobile.changePage("#ranking",{transition:"slide"})
	};

	var pageBeforeShow = function(){
		if(!reload) return;

		pict.removeClass('no-background');
		pict.css('opacity','1');
		board.css('display','none');
		puzzle.attr('data-cols',cols).attr('data-rows',rows);
		with(document.styleSheets[document.styleSheets.length-1]){
			rules[0].style.backgroundImage="url('"+pictURL+"')";
		}
		
		createPuzzle();
	};

	var pageShow=function(){
		if(!reload){
			startTime=new Date().getTime();
			return;
		}else reload=false;

		setTimeout(function(){
			pict.fadeTo(500,0.5,function(){
				pict.toggleClass('no-background',!showBackgroundImage);
				board.fadeIn(1000,function(){
					startTime=new Date().getTime();
					elapsedTime=0;
				});
			});
		},2000);
	};

	var pageBeforeHide=function(){
		elapsedTime+=new Date().getTime()-startTime;
		startTime=0;
	};
	
	var dragOverImage=function(e){
		e.stopPropagation();
		e.preventDefault();
		e.originalEvent.dataTransfer.dropEffect = 'copy'; 
	};
	
	var dropImage=function(e){
		e.stopPropagation();
		e.preventDefault();
		var file = e.originalEvent.dataTransfer.files[0]; 
		if (!file.type.match('image.*')) {
			return;
		}
		var reader = new FileReader();
		reader.onload=function(e2){
			pictURL=e2.target.result;
			elapsedTime=0;
			startTime=0;
			reload=true;
			pageBeforeShow();
			pageShow();
		}
		reader.readAsDataURL(file);
	};
	
	var languageChange=function(){
		puzzleDone.text(i18n.lang.puzzleDone);
	};
	
	var commonReady=function(){
		puzzle.on('pagebeforeshow',pageBeforeShow);
		puzzle.on('pageshow',pageShow);
		puzzle.on('pagebeforehide',pageBeforeHide);
		puzzle.on('dragover',dragOverImage);
		puzzle.on('drop',dropImage);
		moving.on('tap',function(e){
			if(doTapHold){
				tap();
				e.stopPropagation();
			}
		});
		puzzleDone.on('vclick',function(){
			puzzleDone.popup("close");
		});
		puzzleDone.on('popupafterclose',puzzleDoneAfterClose);
		$(window).on('puzzleChange',puzzleChange);
		$(window).on('configChange',configChange);
		$(window).on('languageChange',languageChange);
	};
	
	var deviceReady=function(){
		commonReady();
		dropSound=new Media('/android_asset/www/sounds/drop.mp3',$.noop,$.noop);
		taDa=new Media('/android_asset/www/sounds/taDa.mp3',$.noop,$.noop);
	};
	
	var desktopReady=function(){
		commonReady();
		dropSound=new Audio('sounds/drop.mp3');
		taDa=new Audio('sounds/taDa.mp3');
	};
	
	return {
		deviceReady:deviceReady,
		desktopReady:desktopReady,
		puzzleChange:puzzleChange,
		createPuzzle:createPuzzle,
		isOrdered:isOrdered
	}
};
